package cn.minbb.edu.activities;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.Toolbar;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.style.ForegroundColorSpan;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.drawable.GlideDrawable;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;

import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.edu.R;
import cn.minbb.edu.app.URL;
import cn.minbb.edu.ctrl.UserManager;
import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.model.Course;
import cn.minbb.edu.util.HttpRequestUtil;

public class CourseDetailActivity extends AppCompatActivity {

    @BindView(R.id.toolbar)
    Toolbar toolbar;
    @BindView(R.id.cover)
    ImageView cover;
    @BindView(R.id.name)
    TextView name;
    @BindView(R.id.introduction)
    TextView introduction;
    @BindView(R.id.teacher)
    TextView teacher;

    private ProgressDialog dialog;

    private Course course;

    static {
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_course_detail);
        ButterKnife.bind(this);

        initView();
        loadData();
    }

    public static void start(Context context, Integer id) {
        Intent starter = new Intent(context, CourseDetailActivity.class);
        starter.putExtra("id", id.toString());
        context.startActivity(starter);
    }

    private void initView() {
        setSupportActionBar(toolbar);
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            // 在 ActionBar 中显示 UP 按钮。
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
    }

    private void loadData() {
        showProgress(true);
        Map<String, String> map = new HashMap<>(1);
        map.put("id", getIntent().getStringExtra("id"));
        new HttpRequestUtil(URL.request(URL.URLEnum.COURSE), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                runOnUiThread(() -> {
                    showProgress(false);
                    new AlertDialog.Builder(CourseDetailActivity.this)
                            .setTitle("抱歉").setMessage("加载失败，请重试")
                            .setNegativeButton("取消", (dialogInterface, i) -> CourseDetailActivity.this.finish())
                            .setPositiveButton("重试", (dialogInterface, i) -> loadData())
                            .setCancelable(false).create().show();
                });
            }

            @Override
            public void success(String s) {
                runOnUiThread(() -> {
                    showProgress(false);
                    ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                    course = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), Course.class);
                    Glide.with(CourseDetailActivity.this)
                            .load(course.getCover())
                            .placeholder(R.drawable.banner_loading_16by9)
                            .error(R.drawable.banner_loading_error_16by9)
                            .listener(new RequestListener<String, GlideDrawable>() {
                                @Override
                                public boolean onException(Exception e, String model, Target<GlideDrawable> target, boolean isFirstResource) {
                                    Toast.makeText(getApplicationContext(), "课程封面加载失败", Toast.LENGTH_SHORT).show();
                                    return false;
                                }

                                @Override
                                public boolean onResourceReady(GlideDrawable resource, String model, Target<GlideDrawable> target, boolean isFromMemoryCache, boolean isFirstResource) {
                                    return false;
                                }
                            })
                            .centerCrop()
                            .dontAnimate()
                            .into(cover);
                    SpannableStringBuilder span = new SpannableStringBuilder("缩进" + course.getIntroduction());
                    span.setSpan(new ForegroundColorSpan(Color.TRANSPARENT), 0, 2, Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
                    name.setText(course.getName());
                    introduction.setText(span);
                    teacher.setText("主讲教师：" + course.getVoTeacher().getName());
                });
            }
        }).get(map);
    }

    private void showProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("请稍等");
            dialog.setMessage("加载中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
        }
    }

    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.fab:
                showProgress(true);
                Map<String, Object> params = new HashMap<>(2);
                params.put("user", UserManager.getCurrentUser());
                params.put("course", course);
                new HttpRequestUtil(URL.request(URL.URLEnum.COURSE_USER_ADD), new HttpRequestUtil.RequestCallback() {
                    @Override
                    public void failed(String s) {
                        runOnUiThread(() -> {
                            showProgress(false);
                            Toast.makeText(getApplicationContext(), "请求失败\n" + s, Toast.LENGTH_SHORT).show();
                        });
                    }

                    @Override
                    public void success(String s) {
                        runOnUiThread(() -> {
                            showProgress(false);
                            ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                            Toast.makeText(getApplicationContext(), result.getMessage(), Toast.LENGTH_SHORT).show();
                        });
                    }
                }).post(params);
                break;
            case R.id.learnNow:
                PlayerActivity.start(this, course);
                break;
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
        }
        return super.onOptionsItemSelected(item);
    }
}
