package cn.minbb.edu.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.view.MenuItem;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.edu.R;
import cn.minbb.edu.adapter.MainBasicCourseAdapter;
import cn.minbb.edu.app.URL;
import cn.minbb.edu.ctrl.App;
import cn.minbb.edu.ctrl.UserManager;
import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.model.Course;
import cn.minbb.edu.util.HttpRequestUtil;

public class MyCourseActivity extends AppCompatActivity {

    @BindView(R.id.courseView)
    RecyclerView courseView;
    private ProgressDialog dialog;

    private MainBasicCourseAdapter courseAdapter;
    private List<MainBasicCourseAdapter.ItemBean> courseList = new ArrayList<>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_my_course);
        ButterKnife.bind(this);
        App.setupActionBar(this);
        initView();
        setListener();
        loadData();
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, MyCourseActivity.class);
        context.startActivity(starter);
    }

    private void initView() {
        courseAdapter = new MainBasicCourseAdapter(MyCourseActivity.this, courseList);
    }

    private void setListener() {
        courseAdapter.setOnItemClickListener(position -> CourseDetailActivity.start(MyCourseActivity.this, courseList.get(position).getCourse().getId()));
        courseAdapter.setItemLongClickListener(position -> {
            AlertDialog.Builder builder = new AlertDialog.Builder(MyCourseActivity.this);
            builder.setTitle("删除课程").setMessage("删除课程后在我学习的课程中将不可见，但仍可以再次加入课程。\n确定删除吗？")
                    .setNegativeButton("取消", null)
                    .setPositiveButton("删除", (dialogInterface, i) -> {
                        showProgress(true);
                        Map<String, Object> params = new HashMap<>(2);
                        params.put("user", UserManager.getCurrentUser());
                        params.put("course", courseList.get(position).getCourse());
                        new HttpRequestUtil(URL.request(URL.URLEnum.COURSE_USER_REMOVE), new HttpRequestUtil.RequestCallback() {
                            @Override
                            public void failed(String s) {
                                runOnUiThread(() -> {
                                    showProgress(false);
                                    Toast.makeText(getApplicationContext(), "请求失败\n" + s, Toast.LENGTH_SHORT).show();
                                });
                            }

                            @Override
                            public void success(String s) {
                                runOnUiThread(() -> {
                                    showProgress(false);
                                    ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                                    Toast.makeText(getApplicationContext(), result.getMessage(), Toast.LENGTH_SHORT).show();
                                    courseList.remove(courseList.get(position));
                                    courseAdapter.notifyDataSetChanged();
                                });
                            }
                        }).post(params);
                    }).create().show();
        });
    }

    private void loadData() {
        showProgress(true);
        new HttpRequestUtil(URL.request(URL.URLEnum.COURSE_USER_ALL), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                runOnUiThread(() -> {
                    showProgress(false);
                    Toast.makeText(getApplicationContext(), "请求失败\n" + s, Toast.LENGTH_SHORT).show();
                });
            }

            @Override
            public void success(String s) {
                runOnUiThread(() -> {
                    showProgress(false);
                    courseList.clear();
                    ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                    Collection collection = result.getDataset();
                    for (Object o : collection) {
                        Course course = JSONObject.parseObject(JSONObject.toJSONString(o), Course.class);
                        courseList.add(new MainBasicCourseAdapter.ItemBean(course));
                    }
                    StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL);
                    staggeredGridLayoutManager.setReverseLayout(false);
                    courseView.setLayoutManager(staggeredGridLayoutManager);
                    courseView.setAdapter(courseAdapter);
                });
            }
        }).post(UserManager.getCurrentUser());
    }

    private void showProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("请稍等");
            dialog.setMessage("努力加载中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
        }
        return super.onOptionsItemSelected(item);
    }
}
