package cn.minbb.edu.activities;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.CheckBox;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.edu.R;
import cn.minbb.edu.adapter.StageAdapter;
import cn.minbb.edu.adapter.SubjectListAdapter;
import cn.minbb.edu.app.URL;
import cn.minbb.edu.ctrl.App;
import cn.minbb.edu.ctrl.UserManager;
import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.model.Stage;
import cn.minbb.edu.model.Subject;
import cn.minbb.edu.model.User;
import cn.minbb.edu.util.HttpRequestUtil;

public class SettingsActivity extends AppCompatActivity {

    @BindView(R.id.stage)
    TextView stage;
    @BindView(R.id.subject)
    TextView subject;
    private LayoutInflater inflater;
    private AlertDialog dialog;

    private User user;
    private List<StageAdapter.ItemBean> stageItemBeanList;
    private List<SubjectListAdapter.ItemBean> subjectItemBeanList;
    List<Subject> subjectList;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_settings);
        ButterKnife.bind(this);
        App.setupActionBar(this);
        inflater = LayoutInflater.from(this);
        user = UserManager.getCurrentUser();
        initView();
        loadData();
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, SettingsActivity.class);
        context.startActivity(starter);
    }

    private void initView() {
        Stage voStage = user.getVoStage();
        if (null != voStage) stage.setText(voStage.getName());
        StringBuilder subjects = new StringBuilder();
        subjectList = user.getSubjectList();
        if (null == subjectList) subjectList = new ArrayList<>();
        for (Subject subject : subjectList) {
            subjects.append(subject.getName()).append(" ");
        }
        subject.setText(subjects);
    }

    private void loadData() {
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) finish();
        return super.onOptionsItemSelected(item);
    }

    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.stageView:
                stageItemBeanList = new ArrayList<>();
                new HttpRequestUtil(URL.request(URL.URLEnum.STAGE_ALL), new HttpRequestUtil.RequestCallback() {
                    @Override
                    public void failed(String s) {
                        runOnUiThread(() -> {
                            Toast.makeText(getApplicationContext(), "请求失败\n" + s, Toast.LENGTH_SHORT).show();
                        });
                    }

                    @Override
                    public void success(String s) {
                        runOnUiThread(() -> {
                            ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                            Collection collection = result.getDataset();
                            for (Object o : collection) {
                                Stage stage = JSONObject.parseObject(JSONObject.toJSONString(o), Stage.class);
                                stageItemBeanList.add(new StageAdapter.ItemBean(stage));
                            }
                            View itemView = inflater.inflate(R.layout.dialog_list_view, null, false);
                            ListView stageListView = itemView.findViewById(R.id.list_view);
                            stageListView.setAdapter(new StageAdapter(SettingsActivity.this, stageItemBeanList));
                            AlertDialog.Builder builder = new AlertDialog.Builder(SettingsActivity.this);
                            builder.setTitle("选择学习阶段").setView(itemView);
                            dialog = builder.setNegativeButton("取消", null).create();
                            dialog.show();
                            stageListView.setOnItemClickListener((adapterView, view1, i, l) -> {
                                user.setStageId(stageItemBeanList.get(i).getStage().getId());
                                new HttpRequestUtil(URL.request(URL.URLEnum.USER_INFO), new HttpRequestUtil.RequestCallback() {
                                    @Override
                                    public void failed(String s1) {
                                        runOnUiThread(() -> {
                                            Toast.makeText(getApplicationContext(), "设置失败，请重试\n" + s1, Toast.LENGTH_SHORT).show();
                                        });
                                    }

                                    @Override
                                    public void success(String s1) {
                                        runOnUiThread(() -> {
                                            stage.setText(stageItemBeanList.get(i).getStage().getName());
                                            if (null != dialog) dialog.dismiss();
                                        });
                                    }
                                }).post(user);
                            });
                        });
                    }
                }).post(null);
                break;
            case R.id.subjectView:
                subjectItemBeanList = new ArrayList<>();
                new HttpRequestUtil(URL.request(URL.URLEnum.SUBJECT_ALL), new HttpRequestUtil.RequestCallback() {
                    @Override
                    public void failed(String s) {
                        runOnUiThread(() -> {
                            Toast.makeText(getApplicationContext(), "请求失败\n" + s, Toast.LENGTH_SHORT).show();
                        });
                    }

                    @Override
                    public void success(String s) {
                        runOnUiThread(() -> {
                            ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                            Collection collection = result.getDataset();
                            for (Object o : collection) {
                                Subject subject = JSONObject.parseObject(JSONObject.toJSONString(o), Subject.class);
                                subjectItemBeanList.add(new SubjectListAdapter.ItemBean(subject));
                            }
                            View itemView = inflater.inflate(R.layout.dialog_list_view, null, false);
                            ListView subjectListView = itemView.findViewById(R.id.list_view);
                            List<Integer> subjectIdList = new ArrayList<>();
                            for (Subject subject : subjectList) {
                                subjectIdList.add(subject.getId());
                            }
                            SubjectListAdapter adapter = new SubjectListAdapter(
                                    SettingsActivity.this, subjectItemBeanList, subjectIdList);
                            subjectListView.setAdapter(adapter);
                            AlertDialog.Builder builder = new AlertDialog.Builder(SettingsActivity.this);
                            builder.setTitle("选择关注学科").setView(itemView);
                            dialog = builder
                                    .setNegativeButton("取消", null)
                                    .setPositiveButton("确定", new DialogInterface.OnClickListener() {
                                        @Override
                                        public void onClick(DialogInterface dialogInterface, int i) {
                                            Map<Integer, String> subjectMap = adapter.getSubjectMap();
                                            // 请求参数
                                            Map<String, Object> params = new HashMap<>(2);
                                            params.put("user", user);
                                            params.put("stage", subjectMap.keySet());
                                            // 请求修改用户关注学科
                                            new HttpRequestUtil(URL.request(URL.URLEnum.USER_STAGE), new HttpRequestUtil.RequestCallback() {
                                                @Override
                                                public void failed(String s) {
                                                    runOnUiThread(() -> Toast.makeText(getApplicationContext(), "请求失败\n" + s, Toast.LENGTH_SHORT).show());
                                                }

                                                @Override
                                                public void success(String s) {
                                                    runOnUiThread(() -> {
                                                        StringBuilder subjectText = new StringBuilder();
                                                        for (Integer key : subjectMap.keySet()) {
                                                            subjectText.append(subjectMap.get(key)).append(" ");
                                                        }
                                                        subject.setText(subjectText);
                                                    });
                                                }
                                            }).post(params);
                                        }
                                    }).create();
                            dialog.show();
                        });
                    }
                }).post(null);
                break;
        }
    }
}
