package cn.minbb.edu.activities.ui.main;

import android.arch.lifecycle.Observer;
import android.arch.lifecycle.ViewModelProviders;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cn.minbb.edu.R;
import cn.minbb.edu.activities.CourseDetailActivity;
import cn.minbb.edu.adapter.MainBasicCourseAdapter;
import cn.minbb.edu.app.URL;
import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.model.Course;
import cn.minbb.edu.model.Subject;
import cn.minbb.edu.util.HttpRequestUtil;

public class PlaceholderFragment extends Fragment {

    private static final String ARG_SECTION_NUMBER = "section_number";
    private static final String SECTION_SUBJECT_ID = "section_subject_id";

    private PageViewModel pageViewModel;

    private Integer subjectId = 0;
    private MainBasicCourseAdapter courseBasicAdapter;
    private List<MainBasicCourseAdapter.ItemBean> courseBasicList = new ArrayList<>();

    public static PlaceholderFragment newInstance(int index, Integer id) {
        PlaceholderFragment fragment = new PlaceholderFragment();
        Bundle bundle = new Bundle();
        bundle.putInt(ARG_SECTION_NUMBER, index);
        bundle.putInt(SECTION_SUBJECT_ID, id);
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    public void setArguments(@Nullable Bundle args) {
        super.setArguments(args);
        if (null != args) subjectId = args.getInt(SECTION_SUBJECT_ID, 0);
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        pageViewModel = ViewModelProviders.of(this).get(PageViewModel.class);
        int index = 1;
        if (getArguments() != null) {
            index = getArguments().getInt(ARG_SECTION_NUMBER);
        }
        pageViewModel.setIndex(index);
        loadCourseData(subjectId);
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.fragment_course_center, container, false);
        RecyclerView sectionCourse = root.findViewById(R.id.section_course);
        // new Stagger 布局管理器，布局管理器所需参数:1.规定显示几列 2.item排列方向
        StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL);
        // 通过布局管理器控制条目排列的顺序  true:反向显示 false:正向显示
        staggeredGridLayoutManager.setReverseLayout(false);
        // 设置布局管理器，参数 StaggeredGridLayoutManager，可以是 RecyclerView 实现和 StaggeredGridView 一样的效果
        sectionCourse.setLayoutManager(staggeredGridLayoutManager);
        courseBasicAdapter = new MainBasicCourseAdapter(getActivity(), courseBasicList);
        courseBasicAdapter.setOnItemClickListener(new MainBasicCourseAdapter.ItemClickListener() {
            @Override
            public void onItemClick(int position) {
                CourseDetailActivity.start(getActivity(), courseBasicList.get(position).getCourse().getId());
            }
        });
        sectionCourse.setAdapter(courseBasicAdapter);
        return root;
    }

    private void loadCourseData(Integer id) {
        Map<String, Integer> idMap = new HashMap<>(1);
        idMap.put("id", id);
        // 加载学科课程
        new HttpRequestUtil(URL.request(URL.URLEnum.SUBJECT), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                FragmentActivity activity = getActivity();
                if (null != activity) {
                    activity.runOnUiThread(() -> {
                        Toast.makeText(activity, "请求失败\n" + s, Toast.LENGTH_SHORT).show();
                    });
                }
            }

            @Override
            public void success(String s) {
                FragmentActivity activity = getActivity();
                if (null != activity) {
                    activity.runOnUiThread(() -> {
                        ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                        Subject subject = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), Subject.class);
                        pageViewModel.setData(subject);

                        pageViewModel.getLiveData().observe(PlaceholderFragment.this, new Observer<Subject>() {
                            @Override
                            public void onChanged(@Nullable Subject subject) {
                                System.out.println("TAB 数据更新 ID = " + id);
                                courseBasicList.clear();
                                if (subject != null) {
                                    List<Course> courses = subject.getVoCourseList();
                                    for (Course course : courses) {
                                        courseBasicList.add(new MainBasicCourseAdapter.ItemBean(course));
                                    }
                                    courseBasicAdapter.notifyDataSetChanged();
                                }
                            }
                        });
                    });
                }
            }
        }).post(idMap);
    }
}