package cn.minbb.edu.adapter;

import android.content.Context;
import android.os.Build;
import android.support.annotation.RequiresApi;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.edu.R;
import cn.minbb.edu.model.Subject;
import lombok.Getter;
import lombok.Setter;

public class SubjectListAdapter extends BaseAdapter {

    private List<ItemBean> itemBeen;
    // 用户已选择的 Subject
    private List<Integer> subjectIdList;
    private LayoutInflater layoutInflater;
    @Getter
    private Map<Integer, String> subjectMap;

    @RequiresApi(api = Build.VERSION_CODES.N)
    public SubjectListAdapter(Context context, List<ItemBean> itemBeen, List<Integer> subjectIdList) {
        layoutInflater = LayoutInflater.from(context);
        this.itemBeen = itemBeen;
        this.subjectIdList = subjectIdList;
        subjectMap = new HashMap<>();
        for (Integer id : subjectIdList) {
            subjectMap.put(id, "");
        }
    }

    @Override
    public int getCount() {
        // 获取数据量
        return itemBeen.size();
    }

    @Override
    public Object getItem(int position) {
        // 获取对应 ID 项对应的 Item
        return itemBeen.get(position);
    }

    @Override
    public long getItemId(int position) {
        // 获取对应项 ID
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder viewHolder;
        if (convertView == null) {
            // 将 XML 转化为 View ，并不涉及到具体的布局，所以第二个参数通常设置为 null
            convertView = layoutInflater.inflate(R.layout.item_subject_list, parent, false);
            viewHolder = new ViewHolder(convertView);
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        // 取出 ItemBean 对象
        ItemBean itemBean = itemBeen.get(position);
        // 设置控件数据
        Subject subject = itemBean.subject;
        viewHolder.subject.setOnCheckedChangeListener((compoundButton, b) -> {
            Subject subject1 = (Subject) compoundButton.getTag();
            if (b) {
                subjectMap.put(subject1.getId(), subject1.getName());
            } else {
                subjectMap.remove(subject1.getId());
            }
        });
        viewHolder.subject.setTag(subject);
        viewHolder.subject.setText(subject.getName());
        if (subjectIdList.contains(subject.getId())) {
            viewHolder.subject.setChecked(true);
        } else {
            viewHolder.subject.setChecked(false);
        }
        return convertView;
    }

    public static class ItemBean {

        @Getter
        @Setter
        private Subject subject;

        public ItemBean(Subject subject) {
            this.subject = subject;
        }
    }

    class ViewHolder {
        @BindView(R.id.subject)
        CheckBox subject;

        ViewHolder(View view) {
            ButterKnife.bind(this, view);
        }
    }
}
