package cn.minbb.edu.ctrl;

import android.content.Context;
import android.content.SharedPreferences;
import android.text.TextUtils;

import com.alibaba.fastjson.JSONObject;

import java.util.List;

import cn.minbb.edu.app.URL;
import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.model.User;
import cn.minbb.edu.util.HttpRequestUtil;

public class UserManager {

    private static final String PREFERENCE_USER = "user";

    /**
     * 用户登录
     * 将用户写入 本地文件 / 临时用户变量 保存
     *
     * @param user 用户实体
     */
    public static void userLogin(User user, boolean rememberPassword) {
        if (rememberPassword) {
            SharedPreferences.Editor editorUser = App.getApp().getSharedPreferences(PREFERENCE_USER, Context.MODE_PRIVATE).edit();
            if (user != null) {
                editorUser.putInt("key_user_id", user.getId());
                editorUser.putString("key_user_username", user.getUsername());
                editorUser.putString("key_user_password", user.getPassword());
                editorUser.putString("key_user_name", user.getName());
                editorUser.putString("key_user_avatar", user.getAvatar());
                editorUser.putString("key_user_school", user.getSchool());
            } else {
                editorUser.clear();
            }
            editorUser.apply();
        } else {
            App.user = user;
        }
    }

    /**
     * 用户登出
     * 清除本地的用户文件
     * 清除临时用户变量
     */
    public static void userLogout(User user) {
        App.getApp().getSharedPreferences(PREFERENCE_USER, Context.MODE_PRIVATE).edit().clear().apply();
        App.user = null;
    }

    /**
     * 从本地文件中获取当前用户
     * 注：用户缓存时间过期逻辑未完成
     *
     * @return 用户实体
     */
    public static User getCurrentUser() {
        // 如果临时用户存在则返回临时用户
        User user = App.user;
        if (user != null) {
            return user;
        }
        SharedPreferences sharedPreferencesUser = App.getApp().getSharedPreferences(PREFERENCE_USER, Context.MODE_PRIVATE);
        String username = sharedPreferencesUser.getString("key_user_username", "");
        if (!TextUtils.isEmpty(username)) {
            user = new User();
            user.setId(sharedPreferencesUser.getInt("key_user_id", 0));
            user.setUsername(username);
            user.setPassword(sharedPreferencesUser.getString("key_user_password", ""));
            user.setName(sharedPreferencesUser.getString("key_user_name", ""));
            user.setAvatar(sharedPreferencesUser.getString("key_user_avatar", ""));
            user.setSchool(sharedPreferencesUser.getString("key_user_school", ""));
        }
        return user;
    }

    public AutoLoginCallback autoLoginCallback;

    /**
     * 用户自动登录
     *
     * @param autoLoginCallback 登录回调接口
     */
    public static void userAutoLogin(AutoLoginCallback autoLoginCallback) {
        new HttpRequestUtil(URL.request(URL.URLEnum.USER_AUTO_LOGIN), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                autoLoginCallback.failed();
            }

            @Override
            public void success(String s) {
                ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                User user = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), User.class);
                App.user = user;
                userLogin(user, true);
                autoLoginCallback.success();
            }
        }).post(getCurrentUser());
    }

    public interface AutoLoginCallback {
        void success();

        void failed();
    }
}
